<?php

namespace Teknisa\DF\Modules\Saida\NFSe;

use Teknisa\DF\Modules\Saida\NFSe\Utilities\Util;
use Teknisa\Libs\Util\Environment;

class NFSeCampinas {
    const URL_PRODUCAO = "http://issdigital.campinas.sp.gov.br/WsNFe2/LoteRps.jws?wsdl";
    const URL_HOMOLOGACAO = "http://issdigital.campinas.sp.gov.br/WsNFe2/LoteRps.jws?wsdl";

    protected $connection;
    protected $environment;
    protected $newCode;
    
    protected $ambiente; //1 = producao, 2 = homologacao
    
    // Dados para identificar o Certificado Digital
    protected $cdempresa;
    protected $cdinscmuni;

    protected $xmlEnviarLoteRps;
    protected $listaRps = array();
    protected $listaAuxRps = array();
    protected $itensRps = array();
    protected $xmlConsultarLoteRps;
    protected $xmlCancelarNotas;
    protected $listaNotasCancelamento = array();

    protected $publicKey;
    protected $privateKeyId;
    protected $caminhoCertificado;
    protected $linkCertificado;
    protected $linkCertKey;
    protected $caminhoXml;

    protected $senhaCertificadoDigital;

    protected $datasEmissao = array();

    protected $CodCidade;// = '6291';
    protected $CPFCNPJRemetente;
    protected $RazaoSocialRemetente;
    protected $ValorTotalServicos;
    protected $ValorTotalDeducoes;
    protected $IdLote;

    protected $IdRPS;
    protected $Assinatura;
    protected $NumeroRPS;
    protected $InscricaoMunicipalPrestador;
    protected $TipoRPS;// = 'RPS';
    protected $SerieRPS;// = 'NF';
    protected $DataEmissaoRPS;
    protected $SituacaoRPS;// = 'N';
    protected $SeriePrestacao;// = '99';
    protected $InscricaoMunicipalTomador;
    protected $CPFCNPJTomador;
    protected $RazaoSocialTomador;
    protected $TipoLogradouroTomador;
    protected $LogradouroTomador;
    protected $NumeroEnderecoTomador;
    protected $ComplementoEnderecoTomador;
    protected $TipoBairroTomador;
    protected $BairroTomador;
    protected $CidadeTomador;
    protected $CidadeTomadorDescricao;
    protected $CEPTomador;
    protected $EmailTomador;
    protected $CodigoAtividade;
    protected $AliquotaAtividade;
    protected $TipoRecolhimento;
    protected $MunicipioPrestacao;
    protected $MunicipioPrestacaoDescricao;
    protected $Operacao;
    protected $Tributacao;
    protected $ValorPIS;
	protected $ValorCOFINS;
	protected $ValorINSS;
	protected $ValorIR;
	protected $ValorCSLL;
	protected $AliquotaPIS;
	protected $AliquotaCOFINS;
	protected $AliquotaINSS;
	protected $AliquotaIR;
	protected $AliquotaCSLL;
	protected $DescricaoRPS;
	protected $DDDPrestador;
	protected $TelefonePrestador;
	protected $DDDTomador;
	protected $TelefoneTomador;

	/*
	//Inicio Deducoes
	protected $DeducaoPor;
	protected $TipoDeducao;
	protected $CPFCNPJReferencia;
	protected $NumeroNFReferencia;
	protected $ValorTotalReferencia;
	protected $PercentualDeduzir;
	protected $ValorDeduzir;
	//Fim Deducoes

    //Inicio Item
	protected $DiscriminacaoServico;
	protected $Quantidade;
	protected $ValorUnitario;
	protected $ValorTotal;
	protected $Tributavel;
    // Fim Item
    */

	public function __construct($connection, Environment $environment, $newCode, $xmlDir, $certUploadDir = null, $certUploadLink = null, $certKeyLink = null) {
        $this->connection = $connection;
        $this->environment = $environment;
        $this->newCode = $newCode;
        
        //$this->setCaminhoXml($xmlDir);
        if($certUploadDir) {
            $this->setCaminhoCertificado($certUploadDir);
        }
        else {
            $this->linkCertificado = $certUploadLink;
            $this->linkCertKey     = $certKeyLink;
        }
	}

	private function setCaminhoXml($xmlDir) {
	    $xmlDir .= "org_". $this->environment->getNrOrg() . DIRECTORY_SEPARATOR;
	    $this->caminhoXml = $xmlDir;
	    if (!is_dir($this->caminhoXml)) {
            mkdir($this->caminhoXml, 0775, true);
            chmod($this->caminhoXml, 0775);
        }
	}

	private function setCaminhoCertificado($certUploadDir) {
	    if (!is_null($certUploadDir)) {
	        $certUploadDir .= "org_". $this->environment->getNrOrg() . DIRECTORY_SEPARATOR;
	        $this->caminhoCertificado = $certUploadDir;
	        if (!is_dir($this->caminhoCertificado)) {
	            mkdir($this->caminhoCertificado, 0775, true);
	            chmod($this->caminhoCertificado, 0775);
	        }
	    }
	}

	private function gerarXmlEnviarLoteRps() {
        $qtdeRps = count($this->listaRps);
        
        sort($this->datasEmissao, SORT_STRING);
        $dataInicio = array_shift($this->datasEmissao);
        $dataFim = array_pop($this->datasEmissao);
        
		$this->xmlEnviarLoteRps = '<ns1:ReqEnvioLoteRPS xmlns:ns1="http://localhost:8080/WsNFe2/lote" xmlns:tipos="http://localhost:8080/WsNFe2/tp" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://localhost:8080/WsNFe2/lote http://localhost:8080/WsNFe2/xsd/ReqEnvioLoteRPS.xsd">
			<Cabecalho>
				<CodCidade>'.$this->CodCidade.'</CodCidade>
				<CPFCNPJRemetente>'.$this->CPFCNPJRemetente.'</CPFCNPJRemetente>
				<RazaoSocialRemetente>'.$this->RazaoSocialRemetente.'</RazaoSocialRemetente>
				<transacao>true</transacao>
				<dtInicio>'.$dataInicio.'</dtInicio>
				<dtFim>'.$dataFim.'</dtFim>
				<QtdRPS>'.$qtdeRps.'</QtdRPS>
				<ValorTotalServicos>'.$this->ValorTotalServicos.'</ValorTotalServicos>
				<ValorTotalDeducoes>'.$this->ValorTotalDeducoes.'</ValorTotalDeducoes>
				<Versao>1</Versao>
				<MetodoEnvio>WS</MetodoEnvio>
			</Cabecalho>
			<Lote Id="'.$this->IdLote.'">';

			foreach ($this->listaRps as $rps) {
				$this->xmlEnviarLoteRps .= "\n" . $rps;
			}
		
		$this->xmlEnviarLoteRps .= '
			</Lote>
		</ns1:ReqEnvioLoteRPS>';
	}

	private function adicionarRpsEnviarLote() {
        $xmlRps = '
		<RPS Id="'.$this->IdRPS.'">
			<Assinatura>'.$this->Assinatura.'</Assinatura>
			<InscricaoMunicipalPrestador>'.$this->InscricaoMunicipalPrestador.'</InscricaoMunicipalPrestador>
			<RazaoSocialPrestador>'.$this->RazaoSocialRemetente.'</RazaoSocialPrestador>
			<TipoRPS>'.$this->TipoRPS.'</TipoRPS>
			<SerieRPS>'.$this->SerieRPS.'</SerieRPS>
			<NumeroRPS>'.$this->NumeroRPS.'</NumeroRPS>
			<DataEmissaoRPS>'.$this->DataEmissaoRPS.'</DataEmissaoRPS>
			<SituacaoRPS>'.$this->SituacaoRPS.'</SituacaoRPS>
			<SeriePrestacao>'.$this->SeriePrestacao.'</SeriePrestacao>
			<InscricaoMunicipalTomador>'.$this->InscricaoMunicipalTomador.'</InscricaoMunicipalTomador>
			<CPFCNPJTomador>'.$this->CPFCNPJTomador.'</CPFCNPJTomador>
			<RazaoSocialTomador>'.$this->RazaoSocialTomador.'</RazaoSocialTomador>
			<TipoLogradouroTomador>'.$this->TipoLogradouroTomador.'</TipoLogradouroTomador>
			<LogradouroTomador>'.$this->LogradouroTomador.'</LogradouroTomador>
			<NumeroEnderecoTomador>'.$this->NumeroEnderecoTomador.'</NumeroEnderecoTomador>
			<ComplementoEnderecoTomador>'.$this->ComplementoEnderecoTomador.'</ComplementoEnderecoTomador>
			<TipoBairroTomador>'.$this->TipoBairroTomador.'</TipoBairroTomador>
			<BairroTomador>'.$this->BairroTomador.'</BairroTomador>
			<CidadeTomador>'.$this->CidadeTomador.'</CidadeTomador>
			<CidadeTomadorDescricao>'.$this->CidadeTomadorDescricao.'</CidadeTomadorDescricao>
			<CEPTomador>'.$this->CEPTomador.'</CEPTomador>
			<EmailTomador>'.$this->EmailTomador.'</EmailTomador>
			
			<CodigoAtividade>'.$this->CodigoAtividade.'</CodigoAtividade>
			<AliquotaAtividade>'.$this->AliquotaAtividade.'</AliquotaAtividade>
			<TipoRecolhimento>'.$this->TipoRecolhimento.'</TipoRecolhimento>

			<MunicipioPrestacao>'.$this->MunicipioPrestacao.'</MunicipioPrestacao>
			<MunicipioPrestacaoDescricao>'.$this->MunicipioPrestacaoDescricao.'</MunicipioPrestacaoDescricao>
			<Operacao>'.$this->Operacao.'</Operacao>
			<Tributacao>'.$this->Tributacao.'</Tributacao>
			
			<ValorPIS>'.Util::formatarNumero($this->ValorPIS).'</ValorPIS>
			<ValorCOFINS>'.Util::formatarNumero($this->ValorCOFINS).'</ValorCOFINS>
			<ValorINSS>'.Util::formatarNumero($this->ValorINSS).'</ValorINSS>
			<ValorIR>'.Util::formatarNumero($this->ValorIR).'</ValorIR>
			<ValorCSLL>'.Util::formatarNumero($this->ValorCSLL).'</ValorCSLL>
			<AliquotaPIS>'.Util::formatarNumero($this->AliquotaPIS, 4).'</AliquotaPIS>
			<AliquotaCOFINS>'.Util::formatarNumero($this->AliquotaCOFINS, 4).'</AliquotaCOFINS>
			<AliquotaINSS>'.Util::formatarNumero($this->AliquotaINSS, 4).'</AliquotaINSS>
			<AliquotaIR>'.Util::formatarNumero($this->AliquotaIR, 4).'</AliquotaIR>
			<AliquotaCSLL>'.Util::formatarNumero($this->AliquotaCSLL, 4).'</AliquotaCSLL>

			<DescricaoRPS>'.$this->DescricaoRPS.'</DescricaoRPS>
			<DDDPrestador>'.$this->DDDPrestador.'</DDDPrestador>
			<TelefonePrestador>'.$this->TelefonePrestador.'</TelefonePrestador>
			<DDDTomador>'.$this->DDDTomador.'</DDDTomador>
			<TelefoneTomador>'.$this->TelefoneTomador.'</TelefoneTomador>
			
			<Itens>';

        /*
           <Deducoes>
				<DeducaoPor>'.$this->DeducaoPor.'</DeducaoPor>
				<TipoDeducao>'.$this->TipoDeducao.'</TipoDeducao>
				<CPFCNPJReferencia>'.$this->CPFCNPJReferencia.'</CPFCNPJReferencia>
				<NumeroNFReferencia>'.$this->NumeroNFReferencia.'</NumeroNFReferencia>
				<ValorTotalReferencia>'.$this->ValorTotalReferencia.'</ValorTotalReferencia>
				<PercentualDeduzir>'.$this->PercentualDeduzir.'</PercentualDeduzir>
				<ValorDeduzir>'.$this->ValorDeduzir.'</ValorDeduzir>
			</Deducoes>
        */

			foreach ($this->itensRps as $item) {
				$xmlRps .= '
				<Item>
					<DiscriminacaoServico>'.$item["DiscriminacaoServico"].'</DiscriminacaoServico>
					<Quantidade>'.$item["Quantidade"].'</Quantidade>
					<ValorUnitario>'.$item["ValorUnitario"].'</ValorUnitario>
					<ValorTotal>'.$item["ValorTotal"].'</ValorTotal>
					<Tributavel>'.$item["Tributavel"].'</Tributavel>
				</Item>';
			}

			$xmlRps .= '
			</Itens>
		</RPS>';

        $this->listaRps[] = $xmlRps;
	}

    private function getXmlAssinadoEnviarLoteRps() {
        $this->xmlEnviarLoteRps = str_replace("&", "&#38;", $this->xmlEnviarLoteRps);
        $this->substituiCaracteresEspeciais();
        $domXml = new \DOMDocument("1.0", "UTF-8");
        $domXml->loadXML($this->xmlEnviarLoteRps, LIBXML_NOBLANKS);
        $domXml = $this->removerTagsVazias($domXml);
        $domXml->formatOutput = true;
        $xmlTemp = $domXml->saveXML();

        $domXml = new \DOMDocument("1.0", "UTF-8");
        $domXml->loadXML($xmlTemp);
        $domXml->encoding = "UTF-8";
        $domXml = $this->assinaXml($domXml, array("Lote"));
        if ($domXml === false) {
            throw new \Exception("Certificado digital não encontrado!");
        }
        else {
            $this->limparListaRps();
            return $domXml->saveXML();
        }
    }
    
    private function substituiCaracteresEspeciais() {
        $this->xmlEnviarLoteRps = str_replace("á", "a", $this->xmlEnviarLoteRps);
        $this->xmlEnviarLoteRps = str_replace("â", "a", $this->xmlEnviarLoteRps);
        $this->xmlEnviarLoteRps = str_replace("é", "e", $this->xmlEnviarLoteRps);
        $this->xmlEnviarLoteRps = str_replace("í", "i", $this->xmlEnviarLoteRps);
        $this->xmlEnviarLoteRps = str_replace("ó", "o", $this->xmlEnviarLoteRps);
        $this->xmlEnviarLoteRps = str_replace("ú", "u", $this->xmlEnviarLoteRps);
        $this->xmlEnviarLoteRps = str_replace("ç", "c", $this->xmlEnviarLoteRps);
        
        $this->xmlEnviarLoteRps = str_replace("Á", "A", $this->xmlEnviarLoteRps);
        $this->xmlEnviarLoteRps = str_replace("Â", "A", $this->xmlEnviarLoteRps);
        $this->xmlEnviarLoteRps = str_replace("É", "E", $this->xmlEnviarLoteRps);
        $this->xmlEnviarLoteRps = str_replace("Í", "I", $this->xmlEnviarLoteRps);
        $this->xmlEnviarLoteRps = str_replace("Ó", "O", $this->xmlEnviarLoteRps);
        $this->xmlEnviarLoteRps = str_replace("Ú", "U", $this->xmlEnviarLoteRps);
        $this->xmlEnviarLoteRps = str_replace("Ç", "C", $this->xmlEnviarLoteRps);
        
        $this->xmlEnviarLoteRps = str_replace("ã", "a", $this->xmlEnviarLoteRps);
        $this->xmlEnviarLoteRps = str_replace("à", "a", $this->xmlEnviarLoteRps);
        $this->xmlEnviarLoteRps = str_replace("ê", "e", $this->xmlEnviarLoteRps);
        $this->xmlEnviarLoteRps = str_replace("ó", "o", $this->xmlEnviarLoteRps);
        $this->xmlEnviarLoteRps = str_replace("ô", "o", $this->xmlEnviarLoteRps);
        $this->xmlEnviarLoteRps = str_replace("ü", "u", $this->xmlEnviarLoteRps);
        
        $this->xmlEnviarLoteRps = str_replace("Ã", "A", $this->xmlEnviarLoteRps);
        $this->xmlEnviarLoteRps = str_replace("À", "A", $this->xmlEnviarLoteRps);
        $this->xmlEnviarLoteRps = str_replace("Ê", "E", $this->xmlEnviarLoteRps);
        $this->xmlEnviarLoteRps = str_replace("Ó", "O", $this->xmlEnviarLoteRps);
        $this->xmlEnviarLoteRps = str_replace("Ô", "O", $this->xmlEnviarLoteRps);
        $this->xmlEnviarLoteRps = str_replace("Ü", "U", $this->xmlEnviarLoteRps);
        
    }

    private function removerTagsVazias(\DOMDocument $xml) {
        $xpath = new \DOMXPath($xml);
        foreach($xpath->query('//*[not(node())]') as $node) {
            if (!$node->hasAttributes() && $node->nodeName != "InscricaoMunicipalTomador" && $node->nodeName != "DDDTomador" && $node->nodeName != "TelefoneTomador" && $node->nodeName != "DDDPrestador" && $node->nodeName != "TelefonePrestador" && $node->nodeName != "BairroTomador") {
                $node->parentNode->removeChild($node);
            }
        }
        return $xml;
    }

    private function limparListaRps() {
        $this->listaRps = array();
        $this->datasEmissao = array();
    }

    public function consultarLoteRps($lote) {
        $this->cdempresa = $lote["CDEMPLOTERPS"];
        $this->cdinscmuni = str_replace(array(".","-","/"," "), "", $lote["CDINSCMUNI"]);
        
        $this->ambiente = Util::getAmbiente($this->connection, $lote["CDEMPLOTERPS"], $lote["CDINSCMUNI"], $this->environment->getNrOrg());
        if ($this->ambiente == null) $this->ambiente = 2;
        
        //$this->RazaoSocialRemetente = $lote["NMRAZSOCEMPR"];
        $inscricaoJuridica = str_replace(array("-", "/", ".", " "), "", $lote["NRINSJURIEST"]);
        $xml = $this->gerarXmlConsultarLoteRps($inscricaoJuridica, $lote["NRPROTOCOLORPS"]);
        $resultado = $this->consultarLoteRpsWS($xml, $lote["CDEMPLOTERPS"], $lote["NRINSJURIEST"], $lote["NRSEQLOTERPS"]);
 
        if (is_string($resultado)) {
            $domResult = new \DOMDocument("1.0", "UTF-8");
            $domResult->loadXML($resultado);
            $sucesso = $domResult->getElementsByTagName("Sucesso");
    
            if($sucesso->length > 0) {
                if ($domResult->getElementsByTagName("Sucesso")->item(0)->nodeValue == "true") {
    
                    $alertas = $domResult->getElementsByTagName("Alerta");
                    if ($alertas->length > 0) {
    
                        $erros = array();
                        foreach ($alertas as $alerta) {
                            $erros[] = array(
                                "CODIGO" => Util::getChildNodeByName($alerta, "Codigo")->nodeValue,
                                "DESCRICAO" => Util::getChildNodeByName($alerta, "Descricao")->nodeValue
                            );
                        }
                        return $erros;
                    }
                    else {
                        $nodeList = $domResult->getElementsByTagName("ConsultaNFSe");
                        for ($i = 0; $i < $nodeList->length; $i++) {
                            $numeroNota = str_pad(Util::getChildNodeByName($nodeList->item($i), "NumeroNFe")->nodeValue, 9, "0", STR_PAD_LEFT);
                            $codigoVerificacao = substr(Util::getChildNodeByName($nodeList->item($i), "CodigoVerificacao")->nodeValue, 0, 8);
                            $serieRps = Util::getChildNodeByName($nodeList->item($i), "SerieRPS")->nodeValue;
                            $numeroRps = Util::getChildNodeByName($nodeList->item($i), "NumeroRPS")->nodeValue;
                            $numeroRps = str_pad($numeroRps, 15, "0", STR_PAD_LEFT);
                            $dataEmissaoRPS = Util::getChildNodeByName($nodeList->item($i), "DataEmissaoRPS")->nodeValue;
                            $dataEmissaoRPS = date_create_from_format("Y-m-d\TH:i:s", $dataEmissaoRPS);
                            $dataEmissaoRPS = $dataEmissaoRPS->format("d/m/Y");
    
                            Util::atualizarNumeroRps($this->connection, $serieRps, $numeroRps, $lote["CDEMPLOTERPS"], $lote["CDINSCESTA"], $lote["CDINSCMUNI"], $lote["NRSEQLOTERPS"], $numeroNota, $codigoVerificacao, 4, $dataEmissaoRPS, $dataEmissaoRPS, $this->environment->getNrOrg(), $this->environment->getCdOperador());
    
                            Util::atualizarNotaConsulta($this->connection, $serieRps, $numeroRps, $lote["CDEMPLOTERPS"], $lote["CDINSCESTA"], $lote["CDINSCMUNI"], $lote["NRSEQLOTERPS"], $numeroNota, $this->environment->getNrOrg(), $this->environment->getCdOperador());
                            
                            Util::inserirArquivoXml($this->connection, $serieRps, $numeroRps, $lote["CDEMPLOTERPS"], $lote["CDINSCESTA"], $lote["CDINSCMUNI"], $lote["NRSEQLOTERPS"], "A", $resultado, $this->environment->getNrOrg(), $this->environment->getCdOperador());
                        }
                        
                        Util::atualizarLote($this->connection, $lote["CDEMPLOTERPS"], $lote["CDINSCESTA"], $lote["CDINSCMUNI"], $lote["NRSEQLOTERPS"], 4, $lote["NRPROTOCOLORPS"], $this->environment->getNrOrg(), $this->environment->getCdOperador());
                        
                        return true;
                    }
                }
                else {
                    $chave = "NUMERORPS".$lote["CDEMPLOTERPS"].$lote["NRINSJURIEST"].$lote["CDSERIERPS"];
                    $nrRps = $this->newCode->getCodeByTableName($chave, $this->environment->getNrOrg(), 1, 15);
                    $qtdRps = $lote["QTDERPS"];
                    $nrRps = $nrRps - ($qtdRps + 1);
                    
                    Util::atualizarNovoCodigo($this->connection, $lote["CDEMPLOTERPS"], $lote["NRINSJURIEST"], $lote["CDSERIERPS"],  $nrRps, $this->environment->getNrOrg());
                    
                    Util::atualizarSituacaoNumeroRpsPeloLote($this->connection, $lote["CDSERIERPS"], $lote["CDEMPLOTERPS"], $lote["CDINSCESTA"], $lote["CDINSCMUNI"], $lote["NRSEQLOTERPS"], 3, $this->environment->getNrOrg(), $this->environment->getCdOperador());
                    
                    Util::atualizarLote($this->connection, $lote["CDEMPLOTERPS"], $lote["CDINSCESTA"], $lote["CDINSCMUNI"], $lote["NRSEQLOTERPS"], 3, $lote["NRPROTOCOLORPS"], $this->environment->getNrOrg(), $this->environment->getCdOperador());
    
                    $erros = array();
                    $nodeList = $domResult->getElementsByTagName("Erro");
                    foreach ($nodeList as $erro) {
                        $erros[] = array(
                            "CODIGO" => Util::getChildNodeByName($erro, "Codigo")->nodeValue,
                            "DESCRICAO" => Util::getChildNodeByName($erro, "Descricao")->nodeValue
                        );
                    }
                    return $erros;
                }
            }
            else {
                $chave = "NUMERORPS".$lote["CDEMPLOTERPS"].$lote["NRINSJURIEST"].$lote["CDSERIERPS"];
                $nrRps = $this->newCode->getCodeByTableName($chave, $this->environment->getNrOrg(), 1, 15);
                $qtdRps = $lote["QTDERPS"];
                
                $nrRps = $nrRps - ($qtdRps + 1);
                    
                Util::atualizarNovoCodigo($this->connection, $lote["CDEMPLOTERPS"], $lote["NRINSJURIEST"], $lote["CDSERIERPS"],  $nrRps, $this->environment->getNrOrg());
                    
                Util::atualizarSituacaoNumeroRpsPeloLote($this->connection, $lote["CDSERIERPS"], $lote["CDEMPLOTERPS"], $lote["CDINSCESTA"], $lote["CDINSCMUNI"], $lote["NRSEQLOTERPS"], 3, $this->environment->getNrOrg(), $this->environment->getCdOperador());
                
                Util::atualizarLote($this->connection, $lote["CDEMPLOTERPS"], $lote["CDINSCESTA"], $lote["CDINSCMUNI"], $lote["NRSEQLOTERPS"], 3, $lote["NRPROTOCOLORPS"], $this->environment->getNrOrg(), $this->environment->getCdOperador());

                $erros = array();
                $nodeList = $domResult->getElementsByTagName("Erro");
                foreach ($nodeList as $erro) {
                    $erros[] = array(
                        "CODIGO" => Util::getChildNodeByName($erro, "Codigo")->nodeValue,
                        "DESCRICAO" => Util::getChildNodeByName($erro, "Descricao")->nodeValue
                    );
                }
                return $erros;
            }
        }
        else {
            return false;
        }
    }

    private function consultarLoteRpsWS($xml, $cdempresa, $inscricao, $numeroLote) {
        $nomeFuncao = "consultarLote";
        $prefixoArquivo = $cdempresa.$inscricao.$numeroLote."_".$nomeFuncao."_";
        //file_put_contents($this->caminhoXml . $prefixoArquivo."env.xml", $xml);
        $retorno = $this->enviarParaWebservice($nomeFuncao, $xml);
        //file_put_contents($this->caminhoXml . $prefixoArquivo."resp.xml", $retorno);
        return $retorno;
    }
    
    private function enviarParaWebservice($nomeFuncao, $conteudo) {
        if($this->caminhoCertificado) {
            $caminho = $this->caminhoCertificado . $this->cdempresa . "_" . $this->cdinscmuni . DIRECTORY_SEPARATOR . "certKey.pem";
            $unlink  = false;
        }
        else {
            $certKey = $this->linkCertKey;
            $hash    = uniqid();
            if (is_dir("/tmp")) {
                $caminho= "/tmp/certKey$hash.pem";
            }
            else {
                $caminho = __DIR__ . DIRECTORY_SEPARATOR ."certKey$hash.pem";
            }
            file_put_contents($caminho, file_get_contents($certKey));
            $unlink = true;
        }

        $context = stream_context_create(array(
            'ssl' => array(
                'verify_peer' => false,
                'verify_peer_name' => false,
                'allow_self_signed' => true
            )
        ));

        $options = array(
            'local_cert' => $caminho,
            'trace' => 1,
            'soap_version' => SOAP_1_1,
            "stream_context" => $context,
            'exceptions' => true
        );

        $client = new \SoapClient(self::URL_PRODUCAO, $options);
        $resultado = $client->__soapCall($nomeFuncao, array($conteudo));
        if($unlink) unlink($caminho);
        return $resultado;
    }

    public function cancelarNotas($notas, $cdmotivo, $motivo, $cdmunicibge, $cdmunicsiafi) {
        $this->cdempresa = $notas[0]["CDEMPRESA"];
        $this->cdinscmuni = str_replace(array(".","-","/"," "), "", $notas[0]["CDINSCMUNI"]);
        
        $this->ambiente = Util::getAmbiente($this->connection, $notas[0]["CDEMPRESA"], $notas[0]["CDINSCMUNI"], $this->environment->getNrOrg());
        if ($this->ambiente == null) $this->ambiente = 2;

        $this->senhaCertificadoDigital = Util::getSenhaCertificadoDigital($this->connection, $notas[0]["CDEMPRESA"], $notas[0]["CDINSCMUNI"], $this->environment->getNrOrg());
        
        //$this->RazaoSocialRemetente = $notas[0]["NMRAZSOCEMPR"];
        $this->CPFCNPJRemetente = $notas[0]["NRINSJURIEST"];
        foreach ($notas as $nota) {
            $this->adicionarNotaCancelamento($nota["NRNOTAFISC"], $nota["CDINSCMUNI"], $nota["CDVERIFNFSE"], $motivo);
        }
        $this->gerarXmlCancelarNotas($cdmunicsiafi);
        $xml = $this->getXmlAssinadoCancelarNotas();
        $resultado = $this->cancelarNotasWS($xml, $notas[0]["CDEMPRESA"], $notas[0]["NRINSJURIEST"]);

        if (is_string($resultado)) {
            $domResult = new \DOMDocument("1.0", "UTF-8");
            $domResult->loadXML($resultado);
            if ($domResult->getElementsByTagName("Sucesso")->item(0)->nodeValue == "true") {
                foreach ($notas as $nota) {
                    Util::inserirNotaCancelada($this->connection, "S", "C", $motivo, "C", $nota["CDFILIAL"], $nota["NRLANCTONF"], $this->environment->getNrOrg(), $this->environment->getCdOperador(), $this->newCode);
                    
                    Util::atualizarNotaCancelada($this->connection, "N", $nota["CDFILIAL"], $nota["NRLANCTONF"], $this->environment->getNrOrg());
                }
                return true;
            }
            else {
                $erros = array();
                $nodeList = $domResult->getElementsByTagName("Erro");
                foreach ($nodeList as $erro) {
                    $erros[] = array(
                        "CODIGO" => Util::getChildNodeByName($erro, "Codigo")->nodeValue,
                        "DESCRICAO" => Util::getChildNodeByName($erro, "Descricao")->nodeValue
                    );
                }
                return $erros;
            }
        }
        else {
            return false;
        }
    }

    private function adicionarNotaCancelamento($nrnotafisc, $cdinscmuni, $codigoVerificacao, $motivo) {
        $xml = '
        <Nota Id="NFSE_'.$nrnotafisc.'">
			<InscricaoMunicipalPrestador>'.str_replace(array("/","-","."), "", $cdinscmuni).'</InscricaoMunicipalPrestador>
			<NumeroNota>'.$nrnotafisc.'</NumeroNota>
			<CodigoVerificacao>'.$codigoVerificacao.'</CodigoVerificacao>
			<MotivoCancelamento>'.$motivo.'</MotivoCancelamento>
		</Nota>';

        $this->listaNotasCancelamento[] = $xml;
    }

    private function gerarXmlCancelarNotas($cdmunicsiafi) {
        $this->xmlCancelarNotas = '<?xml version="1.0" encoding="UTF-8"?>
        <ns1:ReqCancelamentoNFSe xmlns:ns1="http://localhost:8080/WsNFe2/lote" xmlns:tipos="http://localhost:8080/WsNFe2/tp" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://localhost:8080/WsNFe2/lote http://localhost:8080/WsNFe2/xsd/ReqCancelamentoNFSe.xsd">
            <Cabecalho>
                <CodCidade>'.$cdmunicsiafi.'</CodCidade>
                <CPFCNPJRemetente>'.$this->CPFCNPJRemetente.'</CPFCNPJRemetente>
                <transacao>true</transacao>
                <Versao>1</Versao>
            </Cabecalho>
            <Lote Id="LOTE_1">';

                foreach ($this->listaNotasCancelamento as $nota) {
                    $this->xmlCancelarNotas .= "\n" . $nota;
                }

            $this->xmlCancelarNotas .= '
            </Lote>
        </ns1:ReqCancelamentoNFSe>';
    }

    private function getXmlAssinadoCancelarNotas() {
        $this->xmlCancelarNotas = str_replace("&", "&#38;", $this->xmlCancelarNotas);

        $domXml = new \DOMDocument("1.0", "UTF-8");
        $domXml->loadXML($this->xmlCancelarNotas, LIBXML_NOBLANKS);
        $domXml = $this->removerTagsVazias($domXml);
        $domXml->formatOutput = true;
        $xmlTemp = $domXml->saveXML();

        $domXml = new \DOMDocument("1.0", "UTF-8");
        $domXml->loadXML($xmlTemp);
        $domXml->encoding = "UTF-8";
        $domXml = $this->assinaXml($domXml);
        if ($domXml === false) {
            throw new \Exception("Certificado digital não encontrado!");
        }
        else {
            $this->listaNotasCancelamento = array();
            return $domXml->saveXML();
        }
    }

    private function cancelarNotasWS($xml, $cdempresa, $inscricao) {
        $nomeFuncao = "cancelar";
        $dataAtual = date_create();
        $dataAtual = $dataAtual->format("Y-m-d_H-i-s");
        $prefixoArquivo = $cdempresa.$inscricao.$dataAtual."_".$nomeFuncao."_";
        //file_put_contents($this->caminhoXml . $prefixoArquivo."env.xml", $xml);
        $resultado = $this->enviarParaWebservice($nomeFuncao, $xml);
        //file_put_contents($this->caminhoXml . $prefixoArquivo."resp.xml", $resultado);
        return $resultado;
    }

    private function gerarXmlConsultarLoteRps($inscricao, $protocolo) {
        $this->xmlConsultarLoteRps = '<?xml version="1.0" encoding="UTF-8"?>
<ns1:ReqConsultaLote xmlns:ns1="http://localhost:8080/WsNFe2/lote" xmlns:tipos="http://localhost:8080/WsNFe2/tp" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://localhost:8080/WsNFe2/lote http://localhost:8080/WsNFe2/xsd/ReqConsultaLote.xsd">
  <Cabecalho>
    <CodCidade>6291</CodCidade>
    <CPFCNPJRemetente>'.$inscricao.'</CPFCNPJRemetente>
    <Versao>1</Versao>
    <NumeroLote>'.$protocolo.'</NumeroLote>
  </Cabecalho>
</ns1:ReqConsultaLote>';
        return $this->xmlConsultarLoteRps;
    }

    public function enviarLoteRps($notas) {
        $erros = array("TIPO" => 1);
        $this->listaAuxRps = array();

        $nrlanctonf = $notas[0]["NRLANCTONF"];
        $cdfilial = $notas[0]["CDFILIAL"];

        $notafiscal = Util::getCorpoNF($this->connection, $nrlanctonf, $cdfilial, $this->environment->getNrOrg());
        
        $this->cdempresa = $notafiscal["CDEMPRESA"];
        $this->cdinscmuni = str_replace(array(".","-","/"," "), "", $notafiscal["CDINSCMUNIEMP"]);
        
        $this->ambiente = Util::getAmbiente($this->connection, $notafiscal["CDEMPRESA"], $notafiscal["CDINSCMUNIEMP"], $this->environment->getNrOrg());
        if ($this->ambiente == null) $this->ambiente = 2;

        $this->senhaCertificadoDigital = Util::getSenhaCertificadoDigital($this->connection, $notafiscal["CDEMPRESA"], $notafiscal["CDINSCMUNIEMP"], $this->environment->getNrOrg());

        $numeroLote = Util::inserirLoteRps($this->connection, $notafiscal["CDEMPRESA"], $notafiscal["CDINSCESTA"], $notafiscal["CDINSCMUNIEMP"], $notafiscal["NRINSJURIEST"], $this->environment->getNrOrg(), $this->environment->getCdOperador(), $this->newCode);

        $this->CodCidade = "6291";
        $this->IdLote = "LOTE_".$notafiscal["CDEMPRESA"]."_".$numeroLote;
        $this->CPFCNPJRemetente = str_replace(array("/", "-", ".", " "), "", $notafiscal["NRINSJURIEST"]);
        $this->InscricaoMunicipalPrestador = str_replace(array("/", "-", ".", " "), "", $notafiscal["CDINSCMUNIEMP"]);//001897870
        $this->RazaoSocialRemetente = $notafiscal["NMRAZSOCEMIT"];

        $this->ValorTotalServicos = 0.0;
        $this->ValorTotalDeducoes = 0.0;

        foreach ($notas as $nota) {
            $nrlanctonf = $nota["NRLANCTONF"];
            $cdfilial = $nota["CDFILIAL"];
            $notafiscal = Util::getCorpoNF($this->connection, $nrlanctonf, $cdfilial, $this->environment->getNrOrg());

            //VALIDACAO
	        if (empty($notafiscal["CDINSCMUNIEMP"])) {
	            $erros[] = array(
	                "CODIGO" => $notafiscal["NRLANCTONF"],
	                "DESCRICAO" => "Inscrição municipal deve ser relacionada com a empresa informada."
	            );
	        }

            $this->ValorTotalServicos += (float)$notafiscal["VRPRODUTOS"];
        	$this->ValorTotalDeducoes += ((float)$notafiscal["VRNOTAFISC"] - (float)$notafiscal["VRPRODUTOS"]);

            // $this->SerieRPS = $notafiscal["CDSERIERPS"];
            $this->SerieRPS = 'NF';
            
            if (empty($this->NumeroRPS)) {
                $this->NumeroRPS = (int)Util::getProximoCodigoNumeroRps($this->connection, $notafiscal["CDEMPRESA"], $notafiscal["NRINSJURIEST"], $notafiscal["CDSERIERPS"], $this->environment->getNrOrg());
            }
            else {
                $this->NumeroRPS += 1;
            }

            $this->IdRPS = "RPS_" . $this->SerieRPS . "_" . $this->NumeroRPS;
            $this->TipoRPS = "RPS";
            $this->SituacaoRPS = "N";

            $dataAtual = date_create();
            
            if(!isset($this->datasEmissao[0])) $this->datasEmissao[] = $notafiscal["DTENTRSAID"];
            
            $this->datasEmissao[] = $notafiscal["DTENTRSAID"];
            $this->DataEmissaoRPS = $notafiscal["DTENTRSAID"] . "T" . $dataAtual->format("H:i:s");

            $this->SeriePrestacao = "99";
            
            $this->CidadeTomador = empty($notafiscal["CDMUNICSIAFICLIE"]) ? $notafiscal["CDMUNICIPIOCLIE"] : $notafiscal["CDMUNICSIAFICLIE"];
            
            $this->InscricaoMunicipalTomador = str_replace(array(".","-","/"," "), "", $notafiscal["NRINSMUNICLIE"]);
            if (strtoupper($this->InscricaoMunicipalTomador) == "ISENTO" || strtoupper($this->InscricaoMunicipalTomador) == "ISENTA" || $this->CidadeTomador != '6291') {
                $this->InscricaoMunicipalTomador = "";
            }
            $this->CPFCNPJTomador = str_replace(array("/", "-", ".", " "), "", $notafiscal["NRINSJURCLIE"]);
            $this->RazaoSocialTomador = $notafiscal["NMRAZSOCCLIE"];

            $this->TipoLogradouroTomador = "-";
            $this->LogradouroTomador = substr($notafiscal["DSENDECLIE"], 0, 50);
            $this->NumeroEnderecoTomador = "-";
            $this->ComplementoEnderecoTomador = substr($notafiscal["DSCOMPENDCLI"], 0, 30);
            $this->TipoBairroTomador = "BAIRRO";
            $this->BairroTomador = substr($notafiscal["NMBAIRCLIE"], 0, 50);
            $this->CidadeTomadorDescricao = substr($notafiscal["NMMUNICIPIOCLIE"], 0, 50);
            $this->CEPTomador = str_replace(array(".", "-", " "), "", $notafiscal["NRCEPCLIE"]);
            $this->EmailTomador = empty($notafiscal["DSEMAILCLIENFE"]) ? "-" : substr($notafiscal["DSEMAILCLIENFE"], 0, 60);

            $itemProduto = Util::getItemProduto($this->connection, $nrlanctonf, $cdfilial, $this->environment->getNrOrg());

			$cdarvprod = $itemProduto[0]["CDARVPROD"];

            $codTribMuni = Util::getCodTribMuni($this->connection, $notafiscal["CDEMPRESA"], $notafiscal["CDINSCMUNIEMP"], $cdarvprod, $this->environment->getNrOrg());
            //VALIDACAO
            if (!is_array($codTribMuni) || empty($codTribMuni["CDTRIBPRODPADR"])) {
                $erros[] = array(
                    "CODIGO" => $notafiscal["NRLANCTONF"],
                    "DESCRICAO" => "Código da tributação da atividade deve ser informado."
                );
            }
            else {
                $this->CodigoAtividade = $codTribMuni["CDTRIBPRODPADR"];//631190001
            }

            $this->AliquotaAtividade = 0.0;
            $impostoItem = Util::getImpostoItemMaiorAliquota($this->connection, $nrlanctonf, $cdfilial, "31", $this->environment->getNrOrg());
            if (!empty($impostoItem) && !empty($impostoItem["VRALIQITEM"])) {
            	$this->AliquotaAtividade = Util::formatarNumero((float)$impostoItem["VRALIQITEM"], 4);
            }
            
            $this->TipoRecolhimento = "A";
            $this->TipoRecolhimentoAss = "N";
            $this->MunicipioPrestacao = $notafiscal["CDMUNISERVPRES"];
            $this->MunicipioPrestacaoDescricao = $notafiscal["NMMUNISERVPRES"];
            
            // Caso a nota tenha deduções altera a operação para B
            if(((float)$notafiscal["VRNOTAFISC"] - (float)$notafiscal["VRPRODUTOS"]) > 0) {
                $this->Operacao  = "B";
            }
            else $this->Operacao = "A";
            
            if($this->MunicipioPrestacao !== '6291') {
                $this->Tributacao = "E";
            } 
            else {
                $this->Tributacao = "T";
            }

            $this->ValorPIS = 0.0;
            $this->ValorCOFINS = 0.0;
            $this->ValorINSS = 0.0;
            $this->ValorIR = 0.0;
            $this->ValorCSLL = 0.0;

            $this->AliquotaPIS = 0.0;
			$this->AliquotaCOFINS = 0.0;
			$this->AliquotaINSS = 0.0;
			$this->AliquotaIR = 0.0;
			$this->AliquotaCSLL = 0.0;

			$impostos = Util::getImpostos($this->connection, $nrlanctonf, $cdfilial, $this->environment->getNrOrg());

            foreach ($impostos as $imp) {

                if (in_array($imp["CDIMPOSTONFE"], array("50", "55", "56")) && $imp["IDTRATIMPO"] == "R") {
                	if ((int)$imp["VRIMPOSTOIT"] > 0) {
                        $this->ValorPIS = (float)$imp["VALORIMPOSTO"];
                        $impostoItem = Util::getImpostoItemMaiorAliquota($this->connection, $nrlanctonf, $cdfilial, $imp["CDIMPOSTONFE"], $this->environment->getNrOrg());
			            if (!empty($impostoItem) && !empty($impostoItem["VRALIQITEM"])) {
                            $this->AliquotaPIS = (float)$impostoItem["VRALIQITEM"];
			            }
                	}
                }
                else if (in_array($imp["CDIMPOSTONFE"], array("60", "65", "66")) && $imp["IDTRATIMPO"] == "R") {
                	if ((int)$imp["VRIMPOSTOIT"] > 0) {
	                    $this->ValorCOFINS = (float)$imp["VALORIMPOSTO"];
                        $impostoItem = Util::getImpostoItemMaiorAliquota($this->connection, $nrlanctonf, $cdfilial, $imp["CDIMPOSTONFE"], $this->environment->getNrOrg());
			            if (!empty($impostoItem) && !empty($impostoItem["VRALIQITEM"])) {
			            	$this->AliquotaCOFINS = (float)$impostoItem["VRALIQITEM"];
			            }
                	}
                }
                else if ($imp["CDIMPOSTONFE"] == "42" && $imp["IDTRATIMPO"] == "R") {
                	if ((int)$imp["VRIMPOSTOIT"] > 0) {
	                    $this->ValorINSS = (float)$imp["VALORIMPOSTO"];
                        $impostoItem = Util::getImpostoItemMaiorAliquota($this->connection, $nrlanctonf, $cdfilial, $imp["CDIMPOSTONFE"], $this->environment->getNrOrg());
			            if (!empty($impostoItem) && !empty($impostoItem["VRALIQITEM"])) {
			            	$this->AliquotaINSS = (float)$impostoItem["VRALIQITEM"];
			            }
                	}
                }
                else if (in_array($imp["CDIMPOSTONFE"], array("21", "22", "25")) && $imp["IDTRATIMPO"] == "R") {
                	if ((int)$imp["VRIMPOSTOIT"] > 0) {
	                    $this->ValorIR = (float)$imp["VALORIMPOSTO"];
			            $impostoItem = Util::getImpostoItemMaiorAliquota($this->connection, $nrlanctonf, $cdfilial, $imp["CDIMPOSTONFE"], $this->environment->getNrOrg());
			            if (!empty($impostoItem) && !empty($impostoItem["VRALIQITEM"])) {
			            	$this->AliquotaIR = (float)$impostoItem["VRALIQITEM"];
			            }
                	}
                }
                else if (in_array($imp["CDIMPOSTONFE"], array("70", "75", "76")) && $imp["IDTRATIMPO"] == "R") {
                	if ((int)$imp["VRIMPOSTOIT"] > 0) {
	                    $this->ValorCSLL = (float)$imp["VALORIMPOSTO"];
			            $impostoItem = Util::getImpostoItemMaiorAliquota($this->connection, $nrlanctonf, $cdfilial, $imp["CDIMPOSTONFE"], $this->environment->getNrOrg());
			            if (!empty($impostoItem) && !empty($impostoItem["VRALIQITEM"])) {
			            	$this->AliquotaCSLL = (float)$impostoItem["VRALIQITEM"];
			            }
                	}
                }
                else if (in_array($imp["CDIMPOSTONFE"], array("30", "31", "32", "35", "36"))) {
                    if ($imp["IDTRATIMPO"] == "R") {
                        $this->TipoRecolhimento = "R";
                        $this->TipoRecolhimentoAss = "S";
                    }
                }
            }
            
            //VALIDACAO
            if (count($impostos) == 0) {
                $erros[] = array(
                    "TIPODEERRO" => "5",
                    "CODIGO" => $nota["NRLANCTONF"],
                    "DESCRICAO" => "Impostos da nota devem ser informados."
                );
            }
            $descricaoProd   = "";
            $descricaoRps    = "";
            $dadosAdicionais = Util::getDadosAdicionais($this->connection, $nrlanctonf, $cdfilial, $this->environment->getNrOrg());
    
            if(isset($dadosAdicionais[0])) {
                foreach($dadosAdicionais as $dadoAdicional) {
                    $descricaoRps .= $dadoAdicional["DSDADOADIC"]."\r\n";
                }
            }
            else {
                foreach ($itemProduto as $linha) {
                    $descricaoProd .= $linha["DSPRODEMIS"] . "\r\n";
                }
    			/*$erros[] = array(
                    "TIPODEERRO" => "5",
                    "CODIGO" => $nota["NRLANCTONF"],
                    "DESCRICAO" => "Dados Adicionais devem ser informados."
                );
                return $erros;*/
            }

            /*$itemEncargo = Util::getItemEncargo($this->connection, $nrlanctonf, $cdfilial, $this->environment->getNrOrg());
            $descricaoEncargo = "";
            foreach ($itemEncargo as $item) {
                $descricaoEncargo .= $item["DESCRICAO"]."\n";
            }

            $descricaoDadosAdic = "";
            $dadosAdicionais = Util::getDadosAdicionais($this->connection, $nrlanctonf, $cdfilial, $this->environment->getNrOrg());
            foreach ($dadosAdicionais as $item) {
                $descricaoDadosAdic .= $item["DSDADOADIC"];
            }*/

            $this->DescricaoRPS = substr(trim($descricaoRps), 0, 1500);// . $descricaoEncargo . $descricaoDadosAdic);
     
            $this->DDDPrestador = "";
			$this->TelefonePrestador = "";
            $telefone = str_replace(array("(", ")", "-", " "), "", $notafiscal["NRTELEEMIT"]);
            if (strlen($telefone) == 11) {
            	$this->DDDPrestador = substr($telefone, 0, 3);
            	$this->TelefonePrestador = substr($telefone, 3);
            }
            else if (strlen($telefone) == 10) {
            	$this->DDDPrestador = substr($telefone, 0, 2);
            	$this->TelefonePrestador = substr($telefone, 2);
            }
            $this->DDDTomador = "";
			$this->TelefoneTomador = "";
            $telefone = str_replace(array("(", ")", "-", " "), "", $notafiscal["NRTELECLIE"]);
            if (strlen($telefone) == 11) {
            	$this->DDDTomador = substr($telefone, 0, 3);
            	$this->TelefoneTomador = substr($telefone, 3);
            }
            else if (strlen($telefone) == 10) {
            	$this->DDDTomador = substr($telefone, 0, 2);
            	$this->TelefoneTomador = substr($telefone, 2);
            }

            $this->itensRps = array();
			foreach ($itemProduto as $item) {
				$discriminacaoServico = substr(trim($item["DSPRODEMIS"]), 0, 80);
				$quantidade = (float)$item["QTITEMPROD"];
                $valorUnitario = Util::formatarNumero($item["VRUNITITEM"]);
                $valorTotal = Util::formatarNumero($item["VRTOTAITEM"]);

                $impostoItem = Util::getImpostosItem($this->connection, $nrlanctonf, $cdfilial, $this->environment->getNrOrg(), "31", $item["NRSEQITPROD"]);
                
                $tributavel = 'S';
                if(isset($impostoItem[0])) {
                    $tributavel = ($impostoItem[0]["CDFISCALIT"] == "1") ? "S" : "N";
                }
                else {
                    $dsErro = "O produto ". $item["DSPRODEMIS"]. " não possui impostos relacionados.";
                    $erros[] = array(
                        "TIPODEERRO" => "5",
                        "NRLANCTONF" => $notafiscal["NRLANCTONF"],
	                    "DESCRICAO"  => $dsErro
                    );
                    return $erros;
                }
				
				$this->itensRps[] = array(
					"DiscriminacaoServico" => $discriminacaoServico,
					"Quantidade" 		   => $quantidade,
					"ValorUnitario" 	   => $valorUnitario,
					"ValorTotal" 		   => $valorTotal,
					"Tributavel" 		   => $tributavel
				);
			}

            $this->ValorTotalServicos = Util::formatarNumero($this->ValorTotalServicos);
            $this->ValorTotalDeducoes = Util::formatarNumero($this->ValorTotalDeducoes);
            
            $dataAssinatura = date("Ymd", strtotime($notafiscal["DTENTRSAID"]));
            
            $valorServico =  (string) round($notafiscal["VRPRODUTOS"], 2);
            $verificaDecimal = explode(".", $valorServico);
            if(isset($verificaDecimal[1])) {
                if(strlen($verificaDecimal[1]) == 1) {
                    $valorServico.='0';
                }
            }
            else {
                $valorServico.='.00';
            }

            $assinatura = str_pad($this->InscricaoMunicipalPrestador, 11, "0", STR_PAD_LEFT)
                . str_pad($this->SerieRPS, 5, " ", STR_PAD_RIGHT)
                . str_pad($this->NumeroRPS, 12, "0", STR_PAD_LEFT)
                . $dataAssinatura
                . str_pad($this->Tributacao, 2, " ", STR_PAD_RIGHT)
                . $this->SituacaoRPS
                . $this->TipoRecolhimentoAss
                . str_pad(str_replace(array(",","."), "", $valorServico), 15, "0", STR_PAD_LEFT)
                . "000000000000000"
                //. str_pad(str_replace(".", "", $codTribMuni["CDCLASSERVPADR"]), 10, "0", STR_PAD_LEFT)
                . str_pad(str_replace(array(".","/","-"), "", $this->CodigoAtividade), 10, "0", STR_PAD_LEFT)
                . str_pad($this->CPFCNPJTomador, 14, "0", STR_PAD_LEFT);

            $this->Assinatura = sha1($assinatura);
            
            $this->adicionarRpsEnviarLote();
            
            $this->listaAuxRps[] = array(
                "CDSERIERPS"   => $notafiscal["CDSERIERPS"],
                "NRRPS"        => $this->NumeroRPS,
                "CDEMPLOTERPS" => $notafiscal["CDEMPRESA"],
                "CDINSCESTA"   => $notafiscal["CDINSCESTA"],
                "CDINSCMUNI"   => $notafiscal["CDINSCMUNIEMP"],
                "NRSEQLOTERPS" => $numeroLote,
                "NRINSJURIEST" => $notafiscal["NRINSJURIEST"],
                "CDFILIAL"     => $cdfilial,
                "NRLANCTONF"   => $nrlanctonf
            );
        }
        if (count($erros) > 1) {
            $this->limparListaRps();
            return $erros;
        }
        else {
            $this->gerarXmlEnviarLoteRps();
            $xml = $this->getXmlAssinadoEnviarLoteRps();
            return $this->enviarLoteRpsWS($xml, $notafiscal["CDEMPRESA"], $notafiscal["NRINSJURIEST"], $numeroLote);
        }
    }

    private function enviarLoteRpsWS($xml, $cdempresa, $inscricao, $numeroLote) {
        
        if ($this->ambiente == 1) {
            $nomeFuncao = "enviar";
        }
        else {
            $nomeFuncao = "testeEnviar";
        }
        
        $prefixoArquivo = $cdempresa.$inscricao.$numeroLote."_".$nomeFuncao."_";
        //file_put_contents($this->caminhoXml . $prefixoArquivo."env.xml", $xml);
        
        $resultado = $this->enviarParaWebservice($nomeFuncao, $xml);
        //file_put_contents($this->caminhoXml . $prefixoArquivo."resp.xml", $resultado);
        
        if (is_string($resultado)) {
            
            if (substr($resultado, 0, 1) == "<") {
                $domResult = new \DOMDocument("1.0", "UTF-8");
                $domResult->loadXML($resultado);
    
                if ($domResult->getElementsByTagName("Sucesso")->item(0)->nodeValue == "true") {
    
                    if ($domResult->getElementsByTagName("Assincrono")->item(0)->nodeValue == "N") {
    
                        $nodeList = $domResult->getElementsByTagName("ChaveNFe");
                        for ($i = 0; $i < $nodeList->length; $i++) {
                            $this->listaAuxRps[$i]["NRNOTAFISCSE"] = Util::getChildNodeByName($nodeList->item($i), "NumeroNFe")->nodeValue;
                            $this->listaAuxRps[$i]["CDVERIFNFSE"] = Util::getChildNodeByName($nodeList->item($i), "CodigoVerificacao")->nodeValue;
                        }
                        
                        
                        foreach ($this->listaAuxRps as $item) {
                            $numeroRPS = str_pad($item["NRRPS"], 15, "0", STR_PAD_LEFT);
                            
      
                            Util::inserirNumeroRps($this->connection, $item["NRSEQLOTERPS"], 4, $item["CDEMPLOTERPS"], $item["CDSERIERPS"], $numeroRPS, $item["CDINSCESTA"], $item["CDINSCMUNI"], $item["NRINSJURIEST"], $item["NRNOTAFISCSE"], $item["CDVERIFNFSE"], $this->environment->getNrOrg(), $this->environment->getCdOperador(), $this->newCode);
                            
                            Util::atualizarNotaConsulta($this->connection, $item["CDSERIERPS"], $numeroRPS, $item["NRSEQLOTERPS"], $item["NRNOTAFISCSE"], "S", $item["CDFILIAL"], $item["NRLANCTONF"], $this->environment->getNrOrg(), $this->environment->getCdOperador());
                        }
    
                        Util::atualizarLote($this->connection, $this->listaAuxRps[0]["CDEMPLOTERPS"], $this->listaAuxRps[0]["CDINSCESTA"], $this->listaAuxRps[0]["CDINSCMUNI"], $this->listaAuxRps[0]["NRSEQLOTERPS"], 4, null, $this->environment->getNrOrg(), $this->environment->getCdOperador());
                        
                        return true;
                    }
                    else {
                        $protocolo = $domResult->getElementsByTagName("NumeroLote")->item(0)->nodeValue;
                        
    
                        foreach ($this->listaAuxRps as $nota) {
                            $numeroRPS = str_pad($nota["NRRPS"], 15, "0", STR_PAD_LEFT);
                            
                            Util::inserirNumeroRps($this->connection, $nota["NRSEQLOTERPS"], 2, $nota["CDEMPLOTERPS"], $nota["CDSERIERPS"], $numeroRPS, $nota["CDINSCESTA"], $nota["CDINSCMUNI"], $nota["NRINSJURIEST"], null, null, $this->environment->getNrOrg(), $this->environment->getCdOperador(), $this->newCode);
                            
                            Util::atualizarNotaTransmissao($this->connection, $nota["CDSERIERPS"], $numeroRPS, $nota["CDEMPLOTERPS"], $nota["CDINSCESTA"], $nota["CDINSCMUNI"], $nota["NRSEQLOTERPS"], $nota["CDFILIAL"], $nota["NRLANCTONF"], $this->environment->getNrOrg(), $this->environment->getCdOperador());
                        }
                        
                        Util::atualizarLote($this->connection, $this->listaAuxRps[0]["CDEMPLOTERPS"], $this->listaAuxRps[0]["CDINSCESTA"], $this->listaAuxRps[0]["CDINSCMUNI"], $this->listaAuxRps[0]["NRSEQLOTERPS"], 2, $protocolo, $this->environment->getNrOrg(), $this->environment->getCdOperador());
                        
                        return true;
                    }
                }
                else {
                    $erros = array("TIPO" => 1);
                    $nodeList = $domResult->getElementsByTagName("Erro");
                    foreach ($nodeList as $erro) {
                        $nodeIdentificacaoRps = $erro->getElementsByTagName("ChaveRPS")->item(0);
           
                        try {
                            $rpsErro = Util::getChildNodeByName($nodeIdentificacaoRps, "NumeroRPS")->nodeValue;
                            foreach($this->listaAuxRps as $item) {
                                if($item["NRRPS"] == $rpsErro) $nota = $item;
                            } 
                        }
                        catch(\Exception $e) {
                            $rpsErro = '';
                            $nota = array('NRLANCTONF' => '');
                        }
                        
                        $erros[] = array(
                            "TIPODEERRO" =>'3',
                            "CODIGO" => Util::getChildNodeByName($erro, "Codigo")->nodeValue,
                            "NRLANCTONF" => $nota["NRLANCTONF"],
                            "DESCRICAO" => Util::getChildNodeByName($erro, "Descricao")->nodeValue,
                            "NRRPS"  => $rpsErro,
                            "XML"    => $xml
                        );
                    }
                    return $erros;
                }
            }
            else {
                $erros = array("TIPO" => 2);
                $erros["XML"] = $xml;

                $erros[] = array(
                    "CODIGO" => "",
                    "DESCRICAO" => $resultado
                );
                return $erros;
            }
        }
        else {
            return false;
        }
    }

    private function assinaXml(\DOMDocument $domXml, array $tags) {
        $local = ($this->caminhoCertificado);
        if($local) {
            $dir = $this->caminhoCertificado . $this->cdempresa . "_" . $this->cdinscmuni;
            if(!is_dir($dir)) return false;
            if((!is_file($dir . DIRECTORY_SEPARATOR . "certificado.pfx")) && (!is_file($dir . DIRECTORY_SEPARATOR . "certificado.p12"))) return false;
            else {
                if(is_file($dir . DIRECTORY_SEPARATOR . "certificado.pfx")) $data = file_get_contents($dir . DIRECTORY_SEPARATOR . "certificado.pfx");
                else if(is_file($dir . DIRECTORY_SEPARATOR . "certificado.p12")) $data = file_get_contents($dir . DIRECTORY_SEPARATOR . "certificado.p12");
            }
        }
        else {
            try{
                $data = file_get_contents($this->linkCertificado);
            }
            catch(\Exception $e) {
                return false;
            }
            if(empty($data)) return false;
        }
        
        openssl_pkcs12_read($data, $cert, $this->senhaCertificadoDigital);

        if (is_null($cert)) {
            return false;
        }
        else {
            //file_put_contents($dir.DIRECTORY_SEPARATOR."certKey.pem", $cert["pkey"]."\r\n".$cert["cert"]);
            //file_put_contents($dir.DIRECTORY_SEPARATOR."priKey.pem", $cert["pkey"]);
            //file_put_contents($dir.DIRECTORY_SEPARATOR."pubKey.pem", $cert["cert"]);

            $this->publicKey = $this->getPublicKey($cert["cert"]);
            $this->privateKeyId = openssl_get_privatekey($cert["pkey"]);

            foreach ($tags as $tag) {
                $domXml = $this->assinaXmlAux($domXml, $tag);
            }

            openssl_free_key($this->privateKeyId);
            return $domXml;
        }

    }

    private function assinaXmlAux(\DOMDocument $domXml, $tag) {
        $nodes = $domXml->getElementsByTagName($tag);
        foreach ($nodes as $node) {
            $root = $node->parentNode;
            $id = trim($node->getAttribute("Id"));
            $dados = $node->C14N(false, false, null, null);
            $hashValue = hash("sha1", $dados, true);
            $digValue = base64_encode($hashValue);

            $signature = $domXml->createElement("Signature");
            $root->appendChild($signature);
            $signature->setAttribute("xmlns", "http://www.w3.org/2000/09/xmldsig#");
            $signature->setAttribute("Id", "Ass_".$id);

            $signedInfo = $domXml->createElement("SignedInfo");
            $signature->appendChild($signedInfo);

            $novaTag = $domXml->createElement("CanonicalizationMethod");
            $signedInfo->appendChild($novaTag);
            $novaTag->setAttribute("Algorithm", "http://www.w3.org/TR/2001/REC-xml-c14n-20010315");

            $novaTag = $domXml->createElement("SignatureMethod");
            $signedInfo->appendChild($novaTag);
            $novaTag->setAttribute("Algorithm", "http://www.w3.org/2000/09/xmldsig#rsa-sha1");

            $reference = $domXml->createElement("Reference");
            $signedInfo->appendChild($reference);
            $reference->setAttribute("URI", "#".$id);

            $transforms = $domXml->createElement("Transforms");
            $reference->appendChild($transforms);

            $novaTag = $domXml->createElement("Transform");
            $transforms->appendChild($novaTag);
            $novaTag->setAttribute("Algorithm", "http://www.w3.org/2000/09/xmldsig#enveloped-signature");

            $novaTag = $domXml->createElement("Transform");
            $transforms->appendChild($novaTag);
            $novaTag->setAttribute("Algorithm", "http://www.w3.org/TR/2001/REC-xml-c14n-20010315");

            $novaTag = $domXml->createElement("DigestMethod");
            $reference->appendChild($novaTag);
            $novaTag->setAttribute("Algorithm", "http://www.w3.org/2000/09/xmldsig#sha1");

            $novaTag = $domXml->createElement("DigestValue", $digValue);
            $reference->appendChild($novaTag);

            $dados = $signedInfo->C14N(false, false, null, null);
            $assinatura = "";
            openssl_sign($dados, $assinatura, $this->privateKeyId);
            $signatureValue = base64_encode($assinatura);


            $novaTag = $domXml->createElement("SignatureValue", $signatureValue);
            $signature->appendChild($novaTag);

            $keyInfo = $domXml->createElement("KeyInfo");
            $signature->appendChild($keyInfo);

            $X509Data = $domXml->createElement("X509Data");
            $keyInfo->appendChild($X509Data);

            $novaTag = $domXml->createElement("X509Certificate", $this->publicKey);
            $X509Data->appendChild($novaTag);
        }
        return $domXml;
    }

    private function getPublicKey($key)
    {
        $data = "";
        $arCert = explode("\n", $key);
        foreach ($arCert as $curData) {
            //remove a tag de inicio e fim do certificado
            if (strncmp($curData, "-----BEGIN CERTIFICATE", 22) != 0 &&
                strncmp($curData, "-----END CERTIFICATE", 20) != 0 ) {
                //carrega o resultado numa string
                $data .= trim($curData);
            }
        }
        return $data;
    }

}